# Define _make_verbose if it doesn't exist (RHEL8)
%{!?_make_verbose:%define _make_verbose V=1 VERBOSE=1}

# See https://github.com/bellard/quickjs/commits/master/
%global qjs_owner    bellard
%global qjs_project  quickjs
%global qjs_commit   19abf1888db5884a5758036ff6e7fa2b340acedc
%global qjs_short    %(c=%{qjs_commit}; echo ${c:0:7})
%global qjs_version  2025.04.26
%global njs_version  0.9.0

Name:           nginx-mod-njs
Version:        %{njs_version}
Release:        1%{?dist}
Summary:        Nginx njs module
License:        2-clause BSD-like license
URL:            https://github.com/nginx/njs
Packager:       Teddysun <i@teddysun.com>
Source0:        https://github.com/nginx/njs/archive/refs/tags/%{version}.tar.gz#/njs-%{version}.tar.gz
Source1:        https://github.com/%{qjs_owner}/%{qjs_project}/archive/%{qjs_commit}/%{qjs_project}-%{qjs_short}.tar.gz
Patch0:         quickjs_quickjs.makefile.patch

BuildRequires:  gcc
BuildRequires:  nginx-mod-devel
BuildRequires:  zlib-devel
BuildRequires:  pcre2-devel
BuildRequires:  libedit-devel
BuildRequires:  libxml2-devel
BuildRequires:  libxslt-devel
BuildRequires:  libatomic

%define WITH_CC_OPT $(echo %{optflags} $(pcre2-config --cflags))
%define WITH_LD_OPT -Wl,-z,relro -Wl,-z,now -Wl,-as-needed
%define bdir %{_builddir}/%{name}-%{version}

%description
Nginx JavaScript, also known as NJS, is a dynamic module for Nginx
that enables the extension of built-in functionality using familiar JavaScript syntax.
The NJS language is a subset of JavaScript, compliant with ES5 (ECMAScript 5.1 Strict Variant) 
with some ES6 (ECMAScript 6) and newer extensions. See compatibility for more details.

%prep
%setup -qcTn %{name}-%{version}

tar xvzfo %{SOURCE0}
ln -s njs-* njs
tar xvzfo %{SOURCE1}
ln -s quickjs-* quickjs

cd %{bdir}/quickjs
%patch0 -p1

%build
_nproc=`getconf _NPROCESSORS_ONLN`
if [ $_nproc -gt 1 ]; then
    _make_opts="-j$_nproc"
fi
cd %{bdir}/quickjs && CFLAGS="%{WITH_CC_OPT} -fPIC" make $_make_opts libquickjs.a
%define __with_quickjs --with-quickjs
cd %{bdir}/njs-%{njs_version} && sed -i.bak 's,-lquickjs.lto ,-lquickjs.lto -latomic ,g' auto/quickjs && sed -i.bak 's,-lquickjs ,-lquickjs -latomic ,g' auto/quickjs && ./configure %{__with_quickjs} --cc-opt="%{WITH_CC_OPT} -I%{bdir}/quickjs/" --ld-opt="%{WITH_LD_OPT} -L%{bdir}/quickjs/" && make $_make_opts njs && mv build build-cli
cd %{bdir}

%undefine _strict_symbol_defs_build
cp -a "%{_nginx_srcdir}" "%{_nginx_buildsrcdir}"
cd "%{_nginx_buildsrcdir}"
./configure --with-compat --with-stream --add-dynamic-module=$(realpath %{_nginx_modsrcdir}/njs-%{njs_version}/nginx) --builddir=$(realpath %{_nginx_modbuilddir}) \
            --with-cc-opt="%{WITH_CC_OPT} -I%{bdir}/quickjs/" \
            --with-ld-opt="%{WITH_LD_OPT} -L%{bdir}/quickjs/"
make %{?_smp_mflags} modules


%install
pushd %{_vpath_builddir}
%{__rm} -rf %{buildroot}
%{__mkdir} -p %{buildroot}%{_bindir}
%{__install} -m755 %{bdir}/njs-%{njs_version}/build-cli/njs %{buildroot}%{_bindir}/

%{__mkdir} -p %{buildroot}%{nginx_moddir}
for so in `find %{bdir}/ -type f -name "*.so"`; do
%{__install} -m755 $so %{buildroot}%{nginx_moddir}
done
popd

%{__install} -dm 0755 %{buildroot}%{nginx_modconfdir}
echo 'load_module "%{nginx_moddir}/ngx_http_js_module.so";'    > %{buildroot}%{nginx_modconfdir}/mod-njs.conf
echo 'load_module "%{nginx_moddir}/ngx_stream_js_module.so";'  >> %{buildroot}%{nginx_modconfdir}/mod-njs.conf

%files
%license njs/LICENSE
%doc njs/CONTRIBUTING.md njs/README.md njs/CODE_OF_CONDUCT.md njs/SECURITY.md njs/SUPPORT.md
%{_bindir}/njs
%{nginx_moddir}/*.so
%{nginx_modconfdir}/mod-njs.conf

%changelog
* Tue May 06 2025 Teddysun <i@teddysun.com> - 0.9.0-1
- update to 0.9.0
- update bundled quickjs to 2025-04-26 release

* Thu Apr 24 2025 Teddysun <i@teddysun.com> - 0.8.10-3
- Rebuild for nginx 1.28.0

* Thu Apr 17 2025 Teddysun <i@teddysun.com> - 0.8.10-2
- Rebuild for nginx 1.27.5

* Fri Apr 11 2025 Teddysun <i@teddysun.com> - 0.8.10-1
- Update version 0.8.10 for nginx 1.27.4

* Sun Mar 16 2025 Teddysun <i@teddysun.com> - 0.8.9-1
- Initial packaging version 0.8.9 for nginx 1.27.4
